logodds.to.stacd <-
function(logodds.M,pa=NULL,pb=NULL){

	#####
	# check for valid input
	#####

	if(!is.numeric(logodds.M) || !(ncol(logodds.M)==3) || !(nrow(logodds.M)==3)){
		stop("logodds.M must be a 3x3 matrix of log odds ratios")
	}

	if(is.null(pa)){
		warning("No allele frequencies given for a. Marginal penetrances computed assuming pa=.5.")
		pa <- .5
	}
	if(is.null(pb)){
		warning("No allele frequencies given for b. Marginal penetrances computed assuming pb=.5.")
		pb <- .5
	}

	#####
	# compute STACD model parameters
	#####

	logodds.V <- matrix(as.vector(t(logodds.M)),ncol=1)

	design.M <- matrix(c(
		1,0,0,0,0,0,0,0,0,
		1,1,0,0,0,0,0,0,0,
		1,2,1,0,0,0,0,0,0,
		1,0,0,1,0,0,0,0,0,
		1,1,0,1,0,1,0,0,0,
		1,2,1,1,0,1,1,0,0,
		1,0,0,2,1,0,0,0,0,
		1,1,0,2,1,1,0,1,0,
		1,2,1,2,1,1,1,1,1),9,9,byrow=TRUE)

	B.V <- as.vector(solve(design.M) %*% logodds.V)
	names(B.V) <- c("mu","alpha_1","alpha_2",
		"beta_1","beta_2","gamma_ab","gamma^*_aab",
		"gamma^*_abb","gamma^*_aabb")

	#####
	# Compute marginal penetrances
	#####

	pen.M <- exp(logodds.M)/(1+exp(logodds.M))
	colnames(pen.M) <- c("AA","Aa","aa")
	rownames(pen.M) <- c("BB","Bb","bb")
	pa.V <- c((1-pa)^2,2*pa*(1-pa),pa^2)
	pb.V <- c((1-pb)^2,2*pb*(1-pb),pb^2)
	p.M <- matrix(pb.V,3,1) %*% matrix(pa.V,1,3)
	marA <- colSums(diag(pb.V)%*%pen.M)
	marB <- rowSums(pen.M%*%diag(pa.V))
	marAB <- sum(p.M*pen.M)

	#####
	# Phenotypic variance of ystar
	#####

	Eu <- logodds.M
	ve <- (pi^2)/3
	vy <- sum(p.M*((Eu-sum(p.M*Eu))^2))+ve


	#####
	# Compute variance decomposition
	#####

	# initialize
	RR <- NA
	param <- rep(NA,8)
	pak <- rep(NA,8)

	### pak sham interaction decomp
	pak[8] <- ((pa*(1-pa)*pb*(1-pb)*(B.V[9]-B.V[8]-B.V[7]+B.V[6]))^2)/vy
	pak[7] <- (2/vy)*(((1-pa)*pa)^2)*pb*(1-pb)*(((1-pb)*B.V[6]-
				(1-pb)*B.V[7]+pb*(B.V[8]-B.V[9]))^2)
	pak[6] <- (2/vy)*(((1-pb)*pb)^2)*pa*(1-pa)*(((1-pa)*B.V[6]-
				(1-pa)*B.V[8]+pa*(B.V[7]-B.V[9]))^2)
	pak[5] <- (4*(1-pa)*pa*(1-pb)*pb*(((1-pa)*(1-pb)*B.V[6]+pb*B.V[8]
				+pa*(B.V[7]-pb*(B.V[7]-B.V[9]+B.V[8])))^2))/vy

	### parameter interaction decomp
	param[8] <- 4*(B.V[9]^2)*(1-pa)*(pa^2)*(1-pb)*(pb^2)/((2-pa)*(2-pb)*vy)
	param[7] <- 2*((1-pa)^2)*pa*(1-pb)*(pb^2)*(((2-pa)*B.V[8]+pa*B.V[9])^2)/
				((2-pa)*(2-pb)*vy)
	param[6] <- 2*((1-pb)^2)*pb*(1-pa)*(pa^2)*(((2-pb)*B.V[7]+pb*B.V[9])^2)/
				((2-pa)*(2-pb)*vy)
	param[5] <- pa*((1-pa)^2)*pb*((1-pb)^2)*(((2-pa)*(2-pb)*B.V[6]+(2-pb)*
				pa*B.V[7]+pb*(2*B.V[8]+pa*B.V[9]-pa*B.V[8]))^2)/
				((2-pa)*(2-pb)*vy)


	### rest of both decomps
	param[4] <- ((pb*(1-pb)*(B.V[5]+pa*(B.V[8]*(2-pa)-B.V[6]*(2-pa)+
				pa*(B.V[9]-B.V[7]))))^2)/vy
	param[3] <- ((2*pb*(1-pb))/vy)*((B.V[4]+pb*B.V[5]+pa*(B.V[6]*
				(1-pb)*(2-pa)+B.V[8]*pb*(2-pa)+pa*(B.V[7]+
				pb*B.V[9]-pb*B.V[7])))^2)
	param[2] <- ((pa*(1-pa)*(B.V[3]+pb*(B.V[7]*(2-pb)-B.V[6]*(2-pb)+
				pb*(B.V[9]-B.V[8]))))^2)/vy
	param[1] <- ((2*pa*(1-pa))/vy)*((B.V[2]+pa*B.V[3]+pb*(B.V[6]*
				(1-pa)*(2-pb)+B.V[7]*pa*(2-pb)+pb*(B.V[8]+
				pa*B.V[9]-pa*B.V[8])))^2)
	pak[1:4] <- param[1:4]

	# verify, get multiple R
	if(all.equal(sum(param),sum(pak)) && all.equal(sum(param),(vy-ve)/vy)){
		RR <- (vy-ve)/vy
	}else{
		warning("Problem in computing variance explained. Multiple R^2 varies by decomposition")
		RR <- NA
	}

	# format
	names(pak) <- c("RRaddA","RRdomA","RRaddB","RRdomB","RRaddadd","RRaddAdomB","RRdomAaddB","RRdomdom")
	names(param) <- c("RRa1", "RRa2", "RRb1", "RRb2", "RRgab", "RRgaab", "RRgabb", "RRgaabb")
	var_ex.L <- list(RR,param,pak)
	names(var_ex.L) <- c("Multiple_R_Sq","Parameter_Decomp","Cockerham_Decomp")


	#####
	# Finish
	#####

	out <- list(B.V,var_ex.L,pen.M,list(A=marA,B=marB),marAB,vy)
	names(out) <- c("Betas","Variance_Explained","Penetrances","Marginals","Prevalence","Var_ystar")

	return(out)
}

