stacd.model <-
function(betas,pa=NA,pb=NA,model="logistic",ve=NA){

	#####
	# Validate input
	#####

	B.V <- betas

	if(is.na(pa)){
		warning("Allele frequency for first locus not given. Defaulting to pa=.5")
		pa <- .5
	}
	if(is.na(pb)){
		warning("Allele frequency for second locus not given. Defaulting to pb=.5")
		pb <- .5
	}

	if(!(length(B.V)==9)){
		stop("B.V must be a vector of the 9 parameter values for the STACD model")
	}
	if(!is.numeric(B.V)){
		stop("The parameter values in B.V must be numeric")
	}
	if(any(is.na(B.V))){
		stop("All parameter values must be specified in B.V")
	} 

	if(length(pa)>1 || length(pb)>1 || !is.numeric(pa) || !is.numeric(pb) || pa > 1 || pa < 0 || pb > 1 || pb < 0){
		stop("pa and pb must be single numeric values giving allele frequencies between 0 and 1 for the two loci")
	}

	if(!(model=="logistic") && !(model=="normal")){
		stop("Model form must be \"logistic\" or \"normal\".")
	}

	#####
	# Conditional Expected Values
	#####

	Ey <- matrix(NA,3,3)
	colnames(Ey) <- c("AA","Aa","aa")
	rownames(Ey) <- c("BB","Bb","bb")
	if(model=="normal"){
		Ey[1,1] <- B.V[1]
		Ey[1,2] <- B.V[1]+B.V[2]
		Ey[1,3] <- B.V[1]+2*B.V[2]+B.V[3]
		Ey[2,1] <- B.V[1]+B.V[4]
		Ey[2,2] <- B.V[1]+B.V[2]+B.V[4]+B.V[6]
		Ey[2,3] <- B.V[1]+2*B.V[2]+B.V[3]+B.V[4]+B.V[6]+B.V[7]
		Ey[3,1] <- B.V[1]+2*B.V[4]+B.V[5]
		Ey[3,2] <- B.V[1]+B.V[2]+2*B.V[4]+B.V[5]+B.V[6]+B.V[8]
		Ey[3,3] <- B.V[1]+2*B.V[2]+B.V[3]+2*B.V[4]+B.V[5]+B.V[6]+B.V[7]+B.V[8]+B.V[9]
	}else if(model=="logistic"){
		Eu <- matrix(NA,3,3)
		Eu[1,1] <- B.V[1]
		Eu[1,2] <- B.V[1]+B.V[2]
		Eu[1,3] <- B.V[1]+2*B.V[2]+B.V[3]
		Eu[2,1] <- B.V[1]+B.V[4]
		Eu[2,2] <- B.V[1]+B.V[2]+B.V[4]+B.V[6]
		Eu[2,3] <- B.V[1]+2*B.V[2]+B.V[3]+B.V[4]+B.V[6]+B.V[7]
		Eu[3,1] <- B.V[1]+2*B.V[4]+B.V[5]
		Eu[3,2] <- B.V[1]+B.V[2]+2*B.V[4]+B.V[5]+B.V[6]+B.V[8]
		Eu[3,3] <- B.V[1]+2*B.V[2]+B.V[3]+2*B.V[4]+B.V[5]+B.V[6]+B.V[7]+B.V[8]+B.V[9]
		Ey <- exp(Eu)/(1+exp(Eu))
	}
	colnames(Ey) <- c("AA","Aa","aa")
	rownames(Ey) <- c("BB","Bb","bb")

	#####
	# Marginals
	#####

	pa.V <- c((1-pa)^2,2*pa*(1-pa),pa^2)
	pb.V <- c((1-pb)^2,2*pb*(1-pb),pb^2)
	p.M <- matrix(pb.V,3,1) %*% matrix(pa.V,1,3)
	marA <- colSums(diag(pb.V)%*%Ey)
	marB <- rowSums(Ey%*%diag(pa.V))
	marAB <- sum(p.M*Ey)

	#####
	# Phenotypic variance
	#####

	if(model=="normal"){
		if(is.na(ve)){
			vy <- 1
			ve <- 1-sum(p.M*((Ey-marAB)^2))
		}else{
			vy <- sum(p.M*((Ey-marAB)^2))+ve			
		}
	}else if(model=="logistic"){
		if(!is.na(ve)){
			warning("ve is ignored for logistic model")
		}
		ve <- (pi^2)/3
		vy <- sum(p.M*((Eu-sum(p.M*Eu))^2))+ve
	}

	#####
	# Variance explained
	#####

	# initialize
	RR <- NA
	param <- rep(NA,8)
	pak <- rep(NA,8)

	### pak sham/cockerham interaction decomp
	pak[8] <- ((pa*(1-pa)*pb*(1-pb)*(B.V[9]-B.V[8]-B.V[7]+B.V[6]))^2)/vy
	pak[7] <- (2/vy)*(((1-pa)*pa)^2)*pb*(1-pb)*(((1-pb)*B.V[6]-
				(1-pb)*B.V[7]+pb*(B.V[8]-B.V[9]))^2)
	pak[6] <- (2/vy)*(((1-pb)*pb)^2)*pa*(1-pa)*(((1-pa)*B.V[6]-
				(1-pa)*B.V[8]+pa*(B.V[7]-B.V[9]))^2)
	pak[5] <- (4*(1-pa)*pa*(1-pb)*pb*(((1-pa)*(1-pb)*B.V[6]+pb*B.V[8]
				+pa*(B.V[7]-pb*(B.V[7]-B.V[9]+B.V[8])))^2))/vy

	### parameter interaction decomp
	sqr <- sqrt(vy*(2-pa)*(2-pb)/(4*(pa^2)*(1-pa)*(pb^2)*(1-pb)))

	param[8] <- 4*B.V[9]^2*(1-pa)*pa^2*(1-pb)*pb^2/((2-pa)*(2-pb)*vy)
	param[7] <- 2*(1-pa)^2*pa*(1-pb)*pb^2*((2-pa)*B.V[8]+pa*B.V[9])^2/((2-pa)*(2-pb)*vy)
	param[6] <- 2*(1-pb)^2*pb*(1-pa)*pa^2*((2-pb)*B.V[7]+pb*B.V[9])^2/((2-pa)*(2-pb)*vy)
	param[5] <- pa*(1-pa)^2*pb*(1-pb)^2*((2-pa)*(2-pb)*B.V[6]+(2-pb)*pa*B.V[7]+pb*(2*B.V[8]+pa*B.V[9]-pa*B.V[8]))^2/((2-pa)*(2-pb)*vy)


	### rest of both decomps
	param[4] <- ((pb*(1-pb)*(B.V[5]+pa*(B.V[8]*(2-pa)-B.V[6]*(2-pa)+
				pa*(B.V[9]-B.V[7]))))^2)/vy
	param[3] <- ((2*pb*(1-pb))/vy)*((B.V[4]+pb*B.V[5]+pa*(B.V[6]*
				(1-pb)*(2-pa)+B.V[8]*pb*(2-pa)+pa*(B.V[7]+
				pb*B.V[9]-pb*B.V[7])))^2)
	param[2] <- ((pa*(1-pa)*(B.V[3]+pb*(B.V[7]*(2-pb)-B.V[6]*(2-pb)+
				pb*(B.V[9]-B.V[8]))))^2)/vy
	param[1] <- ((2*pa*(1-pa))/vy)*((B.V[2]+pa*B.V[3]+pb*(B.V[6]*
				(1-pa)*(2-pb)+B.V[7]*pa*(2-pb)+pb*(B.V[8]+
				pa*B.V[9]-pa*B.V[8])))^2)
	pak[1:4] <- param[1:4]

	# verify, get multiple R
	if(all.equal(sum(param),sum(pak),(vy-ve)/vy)){
		RR <- (vy-ve)/vy
	}else{
		warning("Problem in computing variance explained. Multiple R^2 varies by decomposition")
		RR <- NA
	}

	# format
	names(pak) <- c("RRaddA","RRdomA","RRaddB","RRdomB","RRaddadd","RRaddAdomB","RRdomAaddB","RRdomdom")
	names(param) <- c("RRa1", "RRa2", "RRb1", "RRb2", "RRgab", "RRgaab", "RRgabb", "RRgaabb")
	var_ex.L <- list(RR,param,pak)
	names(var_ex.L) <- c("Multiple_R_Sq","Parameter_Decomp","Cockerham_Decomp")


	#####
	# Finish
	#####

	# label
	names(B.V) <- c("mu","alpha_1","alpha_2",
		"beta_1","beta_2","gamma_ab","gamma^*_aab",
		"gamma^*_abb","gamma^*_aabb")

	if(model=="normal"){
		expect <- list(Ey,marA,marB,marAB)
		names(expect) <- c("Conditional","MarginA","MarginB","Unconditional")
		out <- list(B.V,var_ex.L,expect,vy,ve)
		names(out) <- c("Betas","Variance_Explained","Expected_Values","Var_y","error_variance")
		
	}else if(model=="logistic"){
		out <- list(B.V,var_ex.L,Ey,list(A=marA,B=marB),marAB,vy)
		names(out) <- c("Betas","Variance_Explained","Penetrances","Marginals","Prevalence","Var_ystar")
	}
	return(out)
}

