stacd.power <- function(betas,param,N,nrep,pa,pb,model="logistic",ve=NA,alpha=.05,plotit=TRUE){

	#####
	# Check valid input
	#####
	
	if(!(length(betas)==9)){
		stop("betas must be a vector of the 9 parameter values for the STACD model")
	}
	if(!is.numeric(betas)){
		stop("The parameter values in betas must be numeric")
	}
	if(any(is.na(betas))){
		stop("All parameter values must be specified in betas")
	} 
	
	if(!(param %in% c("mu","alpha_1","alpha_2",
					"beta_1","beta_2","gamma_ab",
					"gamma^*_aab","gamma^*_abb","gamma^*_aabb"))){
		stop("Invalid parameter name; param must be one of \"mu\",
			\"alpha_1\",\"alpha_2\",\"beta_1\",\"beta_2\",
			\"gamma_ab\",\"gamma^*_aab\",\"gamma^*_abb\", or \"gamma^*_aabb\".")
	}
	
	if(!all(round(N)==N)){
		N <- round(N)
		warning(paste("N must be an integer. Rounding to N=",N,sep=""))
	}
	if(!is.numeric(N) || any(N<1) ){
		stop("N must be positive and numeric")
	}
	
	if(!(round(nrep)==nrep)){
		nrep <- round(nrep)
		warning(paste("nrep must be an integer. Rounding to nrep=",nrep,sep=""))
	}
	if(!is.numeric(nrep) || (nrep<1) ){
	stop("nrep must be positive and numeric")
	}
	
	if(is.na(pa)){
		warning("Allele frequency for first locus not given. Defaulting to pa=.5")
		pa <- .5
	}
	if(is.na(pb)){
		warning("Allele frequency for second locus not given. Defaulting to pb=.5")
		pb <- .5
	}
	
	if(length(pa)>1 || length(pb)>1 || !is.numeric(pa) || !is.numeric(pb) || pa > 1 || pa < 0 || pb > 1 || pb < 0){
		stop("pa and pb must be single numeric values giving allele frequencies between 0 and 1 for the two loci")
	}

	if(!(model=="logistic") && !(model=="normal")){
		stop("Model form must be \"logistic\" or \"normal\".")
	}
	
	if(length(alpha)>1 || !is.numeric(alpha) || alpha >= 1 || alpha <= 0){
		stop("alpha must be a single numeric value between 0 and 1 giving the threshold for significance")
	}
	
	if( !is.logical(plotit) || is.na(plotit) || length(plotit)>1 ){
		warning("plotit must be either TRUE or FALSE. Defaulting to TRUE.")
		plotit <- TRUE
	}
	
	if(model=="logistic" && !is.na(ve)){
		warning("ve is ignored for logistic model")
		ve <- NA
	}
	
	#####
	# setup
	#####
	
	paramcol <- which(c("mu","alpha_1","alpha_2",
					"beta_1","beta_2","gamma_ab",
					"gamma^*_aab","gamma^*_abb",
					"gamma^*_aabb")==param)
	outcomes <- matrix(NA,length(N),2)
	outcomes[,1] <- N
	colnames(outcomes) <- c("N","power")
	tests <- matrix(NA,nrep,length(N))
	
	######
	# Simulate
	# - outer loop: sample size
	# - inner loop: sim replications
	######
	
	for(j in 1:length(N)){
		for(i in 1:nrep){
			dat.M <- stacd.data(betas=betas,n=N[j],pa=pa,pb=pb,model=model,ve=ve,verbose=FALSE)
			moddat.M <- cbind(y=dat.M$y,stacd.design(x=dat.M$x,z=dat.M$z))
			if(model=="logistic"){
				fit <- glm(y~.,data=moddat.M,family=binomial("logit"))
			}else if(model=="normal"){
				fit <- lm(y~.,data=moddat.M)
			}
			tests[i,j] <- as.numeric( summary(fit)$coefficients[paramcol,4] <= alpha )
		}
	}
	
	######
	# Format and output
	######
	
	outcomes[,2] <- colMeans( tests )
	if(plotit){
		plot( outcomes , type = "l", main = "Power Analysis" )
	}
	return(outcomes)
}

# eof

  

