varex.to.stacd <-
function(varex,pa,pb,model="logistic",decomp="cockerham",mu=NA,ve=NA,sign=rep(1,8)){

	#####
	# check for valid input
	#####

	if(!(length(varex)==8) || !is.numeric(varex)){
		stop("varex should be a length 8 vector of proportions of variance explained")
	}else if(any(varex<0) || any(varex>1) || sum(varex)>1 ){
		stop("Varience explained cannot be <0 or >1")
	}

	if(length(pa)>1 || length(pb)>1 || !is.numeric(pa) || !is.numeric(pb) || pa > 1 || pa < 0 || pb > 1 || pb < 0){
		stop("pa and pb must be single numeric values giving allele frequencies between 0 and 1 for the two loci")
	}

	if(!(decomp=="cockerham") && !(decomp=="parameter")){
		warning("Variance decomposition must be \"parameter\" or \"cockerham\". Defaulting to \"cockerham\".")
		decomp <- "cockerham"
	}

	if(!(model=="logistic") && !(model=="normal")){
		warning("Model form must be \"logistic\" or \"normal\". Defaulting to \"logistic\".")
		model <- "logistic"
	}

	#####
	# initialize
	#####

	B.V <- rep(NA,9)

	if(model=="logistic"){
		RR <- sum(varex)
		if(!is.na(ve)){
			warning("ve is ignored for logistic model")
		}
		ve <- (pi^2)/3
		vy <- ve/(1-RR)
	}else if(model=="normal"){
		RR <- sum(varex)
		if(is.na(ve)){
			vy <- 1
			ve <- 1-RR
		}else{
			vy <- ve/(1-RR)
		}
	}else{
		stop("Model must be either \"normal\" or \"logistic\".")		
	}

	#####
	# interaction parms
	#####

	if(decomp=="cockerham"){
		# setup terms
		AA <- sign[5]*sqrt(varex[5]*vy)/sqrt(4*pa*(1-pa)*pb*(1-pb))
		AD <- sign[6]*sqrt(varex[6]*vy)/(pb*(1-pb)*sqrt(2*pa*(1-pa)))
		DA <- sign[7]*sqrt(varex[7]*vy)/(pa*(1-pa)*sqrt(2*pb*(1-pb)))
		DD <- sign[8]*sqrt(varex[8]*vy)/(pa*(1-pa)*pb*(1-pb))
		
		# compute
		B.V[6] <- AA - pb*AD - pa*DA + pa*pb*DD
		B.V[7] <- AA + (1-pa)*DA - pb*AD - pb*(1-pa)*DD
		B.V[8] <- AA + (1-pb)*AD - pa*DA - pa*(1-pb)*DD
		B.V[9] <- AA + (1-pa)*DA + (1-pb)*AD + (1-pa)*(1-pb)*DD

	}else if(decomp=="parameter"){
		# setup
		sqr <- sqrt(vy*(2-pa)*(2-pb)/(4*(pa^2)*(1-pa)*(pb^2)*(1-pb)))

		# compute
		B.V[9] <- sign[8]*(sqrt(varex[8])*sqr)
		B.V[8] <- ((1/(2-pa))*(sign[7]*sqrt(2*pa*varex[7]/(1-pa))*sqr-pa*B.V[9]))
		B.V[7] <- ((1/(2-pb))*(sign[6]*sqrt(2*pb*varex[6]/(1-pb))*sqr-pb*B.V[9]))
		B.V[6] <- ((1/((2-pa)*(2-pb)))*
			(sign[5]*sqrt(4*pa*pb*varex[5]/((1-pa)*(1-pb)))*sqr-
			pa*(2-pb)*B.V[7]-pb*(2-pa)*B.V[8]-pa*pb*B.V[9]))
	}else{
		stop("Invalid epistatic variance decomposition method")
	}

	#####
	# dominance terms
	#####

	B.V[3] <- (sign[2]*sqrt(varex[2]*vy)/(pa*(1-pa))-
		pb*(B.V[7]*(2-pb)-B.V[6]*(2-pb)+pb*(B.V[9]-B.V[8])))
	B.V[5] <- (sign[4]*sqrt(varex[4]*vy)/(pb*(1-pb))-
		pa*(B.V[8]*(2-pa)-B.V[6]*(2-pa)+pa*(B.V[9]-B.V[7])))

	#####
	# additive terms
	#####

	B.V[2] <- (sign[1]*sqrt(varex[1]*vy)/sqrt(2*pa*(1-pa))-
		pa*B.V[3]-
		pb*(B.V[6]*(1-pa)*(2-pb)+B.V[7]*pa*(2-pb)+
			pb*(B.V[8]+pa*B.V[9]-pa*B.V[8])))
	B.V[4] <- (sign[3]*sqrt(varex[3]*vy)/sqrt(2*pb*(1-pb))-
		pb*B.V[5]-
		pa*(B.V[6]*(1-pb)*(2-pa)+B.V[8]*pb*(2-pa)+
			pa*(B.V[7]+pb*B.V[9]-pb*B.V[7])))

	#####
	# Back compute other variance decomp
	#####

	if(decomp=="parameter"){
		param <- varex
		pak <- rep(NA,8)
		pak[1:4] <- param[1:4]
		pak[8] <- ((pa*(1-pa)*pb*(1-pb)*(B.V[9]-B.V[8]-B.V[7]+B.V[6]))^2)/vy
		pak[7] <- (2/vy)*(((1-pa)*pa)^2)*pb*(1-pb)*(((1-pb)*B.V[6]-
					(1-pb)*B.V[7]+pb*(B.V[8]-B.V[9]))^2)
		pak[6] <- (2/vy)*(((1-pb)*pb)^2)*pa*(1-pa)*(((1-pa)*B.V[6]-
					(1-pa)*B.V[8]+pa*(B.V[7]-B.V[9]))^2)
		pak[5] <- (4*(1-pa)*pa*(1-pb)*pb*(((1-pa)*(1-pb)*B.V[6]+pb*B.V[8]
					+pa*(B.V[7]-pb*(B.V[7]-B.V[9]+B.V[8])))^2))/vy
	}else if(decomp=="cockerham"){
		pak <- varex
		param <- rep(NA,8)
		param[1:4] <- pak[1:4]
#		sqr <- sqrt(vy*(2-pa)*(2-pb)/(4*(pa^2)*(1-pa)*(pb^2)*(1-pb)))
#		param[8] <- (B.V[9]/sqr)^2
		param[8] <- 4*B.V[9]^2*(1-pa)*pa^2*(1-pb)*pb^2/((2-pa)*(2-pb)*vy)
		param[7] <- 2*(1-pa)^2*pa*(1-pb)*pb^2*((2-pa)*B.V[8]+pa*B.V[9])^2/((2-pa)*(2-pb)*vy)
		param[6] <- 2*(1-pb)^2*pb*(1-pa)*pa^2*((2-pb)*B.V[7]+pb*B.V[9])^2/((2-pa)*(2-pb)*vy)
		param[5] <- pa*(1-pa)^2*pb*(1-pb)^2*((2-pa)*(2-pb)*B.V[6]+(2-pb)*pa*B.V[7]+pb*(2*B.V[8]+pa*B.V[9]-pa*B.V[8]))^2/((2-pa)*(2-pb)*vy)
#		param[7] <- ((1-pa)/(2*pa))*((B.V[8]*(2-pa)+pa*B.V[9])/sqr)^2
#		param[6] <- ((1-pb)/(2*pb))*((B.V[7]*(2-pb)+pb*B.V[9])/sqr)^2
#		param[5] <- (((1-pa)*(1-pb))/(4*pa*pb))*((B.V[6]*(2-pa)*(2-pb)+
#					pa*pb*B.V[9]+pb*(2-pa)*B.V[8]+
#					pa*(2-pb)*B.V[7])/sqr)^2
	}

	# format variance decomps
	names(pak) <- c("RRaddA","RRdomA","RRaddB","RRdomB","RRaddadd","RRaddAdomB","RRdomAaddB","RRdomdom")
	names(param) <- c("RRa1", "RRa2", "RRb1", "RRb2", "RRgab", "RRgaab", "RRgabb", "RRgaabb")
	var_ex.L <- list(RR,param,pak)
	names(var_ex.L) <- c("Multiple_R_Sq","Parameter_Decomp","Cockerham_Decomp")

	#####
	# Conditional Expected Values and Marginals
	# - compute as if mu=0, the add mu later
	#####

	Exb <- matrix(NA,3,3)
	colnames(Exb) <- c("AA","Aa","aa")
	rownames(Exb) <- c("BB","Bb","bb")
	Exb[1,1] <- 0
	Exb[1,2] <- B.V[2]
	Exb[1,3] <- 2*B.V[2]+B.V[3]
	Exb[2,1] <- B.V[4]
	Exb[2,2] <- B.V[2]+B.V[4]+B.V[6]
	Exb[2,3] <- 2*B.V[2]+B.V[3]+B.V[4]+B.V[6]+B.V[7]
	Exb[3,1] <- 2*B.V[4]+B.V[5]
	Exb[3,2] <- B.V[2]+2*B.V[4]+B.V[5]+B.V[6]+B.V[8]
	Exb[3,3] <- 2*B.V[2]+B.V[3]+2*B.V[4]+B.V[5]+B.V[6]+B.V[7]+B.V[8]+B.V[9]

	pa.V <- c((1-pa)^2,2*pa*(1-pa),pa^2)
	pb.V <- c((1-pb)^2,2*pb*(1-pb),pb^2)
	p.M <- matrix(pb.V,3,1) %*% matrix(pa.V,1,3)

	# add mu into expected values
	# by default set mu so E(y) or E(u)=0
	if(is.na(mu)){
		mu <- -sum(p.M*Exb)
	}
	if(model=="normal"){
		Ey <- Exb+mu
	}else if(model=="logistic"){
		Eu <- Exb+mu
		Ey <- exp(Eu)/(1+exp(Eu))
	}
	B.V[1] <- mu
	colnames(Ey) <- c("AA","Aa","aa")
	rownames(Ey) <- c("BB","Bb","bb")

	# marginals
	marA <- colSums(diag(pb.V)%*%Ey)
	marB <- rowSums(Ey%*%diag(pa.V))
	marAB <- sum(p.M*Ey)

	#####
	# Finish
	#####

	# label
	names(B.V) <- c("mu","alpha_1","alpha_2",
		"beta_1","beta_2","gamma_ab","gamma^*_aab",
		"gamma^*_abb","gamma^*_aabb")

	
	if(model=="normal"){
		expect <- list(Ey,marA,marB,marAB)
		names(expect) <- c("Conditional","MarginA","MarginB","Unconditional")
		out <- list(B.V,var_ex.L,expect,vy,ve)
		names(out) <- c("Betas","Variance_Explained","Expected_Values","Var_y","error_variance")
	}else if(model=="logistic"){
		out <- list(B.V,var_ex.L,Ey,list(A=marA,B=marB),marAB,vy)
		names(out) <- c("Betas","Variance_Explained","Penetrances","Marginals","Prevalence","Var_ystar")
	}
	return(out)
}

